module sigctrl ( CLK, RST_X, LAMP_R, LAMP_B, LAMP_Y );
input           CLK, RST_X;
output          LAMP_R, LAMP_B, LAMP_Y;
reg             LAMP_R, LAMP_B, LAMP_Y;

reg   [1:0]     CUR_ST;                     // current status
reg   [1:0]     NXT_ST;                     // next status
reg   [3:0]     TIM;
reg             R_END, B_END, Y_END, SYNCR;

parameter       P_RED_ST    = 2'h0;         //red signal turn on
parameter       P_BLUE_ST   = 2'h1;         //blue signal turn on
parameter       P_YELLOW_ST = 2'h2;         //yellow signal turn on

//-----------------------------------------------
// FF for saving current status (2)
//-----------------------------------------------
always @( posedge CLK or negedge RST_X ) begin
    if ( !RST_X )
        CUR_ST <= P_RED_ST;
    else
        CUR_ST <= NXT_ST;
end

//-----------------------------------------------
// combination circuit for next status (1)
//-----------------------------------------------
always @( CUR_ST or R_END or B_END or Y_END ) begin
    case ( CUR_ST )
        P_RED_ST :
            if ( R_END == 1'b1 ) begin
                NXT_ST = P_BLUE_ST;
                SYNCR  = 1'b1;
            end
            else begin
                NXT_ST = P_RED_ST;
                SYNCR  = 1'b0;
            end
        P_BLUE_ST :
            if ( B_END == 1'b1 ) begin
                NXT_ST = P_YELLOW_ST;
                SYNCR  = 1'b1;
            end
            else begin
                NXT_ST = P_BLUE_ST;
                SYNCR  = 1'b0;
            end
        P_YELLOW_ST :
            if ( Y_END == 1'b1 ) begin
                NXT_ST = P_RED_ST;
                SYNCR  = 1'b1;
            end
            else begin
                NXT_ST = P_YELLOW_ST;
                SYNCR  = 1'b0;
            end
        default: begin
            NXT_ST = P_RED_ST;
            SYNCR  = 1'b1;
        end
    endcase
end

//-----------------------------------------------
// decode for turning on the lamp (3)
//-----------------------------------------------
always @( CUR_ST ) begin
    case ( CUR_ST )
        P_RED_ST : begin
            LAMP_R = 1'b1;
            LAMP_B = 1'b0;
            LAMP_Y = 1'b0;
        end
        P_BLUE_ST : begin
            LAMP_R = 1'b0;
            LAMP_B = 1'b1;
            LAMP_Y = 1'b0;
        end
        P_YELLOW_ST : begin
            LAMP_R = 1'b0;
            LAMP_B = 1'b0;
            LAMP_Y = 1'b1;
        end
        default: begin
            LAMP_R = 1'b0;
            LAMP_B = 1'b0;
            LAMP_Y = 1'b0;
        end
    endcase
end

//-----------------------------------------------
// timer
//-----------------------------------------------
always @( posedge CLK or negedge RST_X ) begin
    if ( !RST_X )
        TIM <= 4'h0;
    else if ( SYNCR == 1'b1 )
        TIM <= 4'h0;
    else
        TIM <= TIM + 4'h1;
end

//-----------------------------------------------
// decode for end time of lamp
//-----------------------------------------------
always @( TIM ) begin
    if ( TIM == 4'd2 ) begin
        R_END = 1'b0;
        B_END = 1'b0;
        Y_END = 1'b1;
    end
    else if ( TIM == 4'd9 ) begin
        R_END = 1'b1;
        B_END = 1'b0;
        Y_END = 1'b0;
    end
    else if ( TIM == 4'd14 ) begin
        R_END = 1'b0;
        B_END = 1'b1;
        Y_END = 1'b0;
    end
    else begin
        R_END = 1'b0;
        B_END = 1'b0;
        Y_END = 1'b0;
    end
end

endmodule
